'use strict';

var postcss = require('postcss');
var unprefixed = postcss.vendor.unprefixed;

var _require = require('./utils.js'),
    getDirRule = _require.getDirRule,
    setRuleDir = _require.setRuleDir,
    rtlifyDecl = _require.rtlifyDecl,
    rtlifyRule = _require.rtlifyRule;

module.exports = postcss.plugin('postcss-rtl', function () {
    return function (css) {

        // Keyframes-animations
        css.walkAtRules(function (rule) {
            var name = rule.params;
            var rtl = void 0;
            var rtlRule = void 0;

            if (rule.name !== unprefixed('keyframes')) return;
            if (name.match(/-ltr|-rtl/)) return;

            rtl = rtlifyRule(rule);

            if (!rtl) return;

            rtlRule = rule.cloneAfter({ params: name + '-rtl' });
            rule.params += '-ltr';

            rtlRule.walkDecls(function (decl) {
                decl.value = rtlifyDecl(decl).value;
            });

            css.walkDecls(function (decl) {
                var callerRule = void 0;
                if (!decl.prop.match(/animation/)) return;
                if (!decl.value.match(new RegExp(name))) return;

                callerRule = decl.parent;
                if (callerRule.selector.match(/\[dir/)) return;
                if (callerRule.selector.match(/\[dir/)) return;

                var rtlDirRule = getDirRule(callerRule, 'rtl');
                var rtlDecl = decl.clone({ value: decl.value.replace(name, name + '-rtl') });
                rtlDirRule.append(rtlDecl);

                setRuleDir(callerRule, 'ltr');
                decl.value = decl.value.replace(name, name + '-ltr');
            });
        });

        // Simple rules (includes rules inside @media-queries)
        css.walkRules(function (rule) {

            if (rule.selector.match(/\[dir/)) return;
            if (rule.parent.type === 'atrule' && rule.parent.name === unprefixed('keyframes')) return;

            var hasSymmetricDecls = false;
            var ltrDecls = [];
            var rtlDecls = [];

            rule.walkDecls(function (decl) {
                var rtl = rtlifyDecl(decl);

                hasSymmetricDecls = hasSymmetricDecls || !rtl;

                if (rtl) {
                    ltrDecls.push(decl);
                    rtlDecls.push(decl.clone(rtl));
                }
            });

            if (ltrDecls.length) {
                (function () {
                    getDirRule(rule, 'rtl').append(rtlDecls);
                    var ltrDirRule = getDirRule(rule, 'ltr');
                    ltrDecls.forEach(function (_decl) {
                        return _decl.moveTo(ltrDirRule);
                    });
                })();
            }

            if (rule.nodes.length === 0) {
                rule.remove();
            } else {
                setRuleDir(rule);
            }
        });
        return false;
    };
});